/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.testsuites;



import java.io.IOException;
import java.util.List;



import org.thermostat.qa.annotations.TestType;
import org.thermostat.qa.annotations.TestTypes;
import org.thermostat.qa.framework.Assert;
import org.thermostat.qa.framework.ThermostatTest;


/**
 * Class ServiceSmokeTests
 * contains basic smoke tests for checking if thermostat service works
 * properly. Starting the agent and storage by command "thermostat service"
 * is checked. 
 *
 */
@TestType(TestTypes.SERVICE_TEST)
public class ServiceSmokeTests extends ThermostatTest
{
    @Override
    protected void setUp()
    {
        // TODO Auto-generated method stub
    }

    @Override
    protected void tearDown()
    {
        // TODO Auto-generated method stub
    }
    
    /**
     * Method getNumberOfAgentsRunning
     * runs the script
     * <a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a> 
     * 
     * 
     * @return
     *      the count of running Thermostat agents
     * @throws IOException
     */
    private int getNumberOfAgentsRunning() throws IOException
    {
      
        List<String> output = runHelperBashScript("list_all_therm_agents.sh");
        Assert.assertNotNull(output, "error running script 'list_all_therm_agents.sh'");
        
        return output.size();
    }
    
    /**
     * Method stopAllThermostatAgents
     * kills all processes that are thermostat agents by running script
     * <a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a> 
     * 
     * @throws IOException
     */
    private void stopAllThermostatAgents() throws IOException
    {
        List<String> output = runHelperBashScript("stop_all_therm_agents.sh");
        Assert.assertNotNull(output, "error running script 'list_all_therm_agents.sh'");        
    }
        
    /**
     * Method getNumberOfMongosRunning
     * runs the script
     * <a href="scripts/ps_mongo_daemon.sh">ps_mongo_daemon.sh</a> 
     * 
     * 
     * @return
     *      the count of running mongo processes
     * @throws IOException
     */
    private int getNumberOfMongosRunning() throws IOException
    {
        List<String> out = runHelperBashScript("ps_mongo_daemon.sh");
        Assert.assertNotNull(out, "script 'ps_mongo_daemon.sh' can't be started");
                
        return out.size();
    }

    /**
     * Method testStartService
     * runs thermostat service and checks if storage and agent are running
     * at the right time. Before the command they should not be running,
     * after the command they should, and after stopping the agent, calling
     * "storage --stop" and waiting for 10s there should be no running db or
     * agents present.
     * 
     * Methods called from this test use the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>,
     * <li><a href="scripts/ps_mongo_daemon.sh">ps_mongo_daemon.sh</a>.
     * <ul>
     * 
     * @throws IOException
     */
    public void testStartService() throws IOException
    {
        //ensure there is no running storage before the test
        Assert.assertEquals(getNumberOfMongosRunning(), 0, "there should be no db running before the test");
        
        //ensure there is no running agent before the test
        Assert.assertEquals(getNumberOfAgentsRunning(), 0, "there should not be any agent running before the test");
        
        //run thermostat service
        new Thread(new ServiceStarter()).start();
        try
        {
            Thread.sleep(FIVE_SEC);
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        
        //detect a running storage
        Assert.assertTrue(getNumberOfMongosRunning()>0, "the db should be running after service started");
        
        //detect a running agent  
        Assert.assertTrue(getNumberOfAgentsRunning()>0, "there should be >0 running thermostat agents");
        
        //stop thermostat agent + storage
        stopAllThermostatAgents();
        Process process = runThermostatInNewProcess("storage", "--stop");
        List<String> processOutput = readProcessOutput(process);
        for (String s : processOutput)
        {
            System.out.println("Service stopper: " + s);
        }
        Assert.assertNotNull(processOutput, "error getting standard output");
        
        //wait till storage+agent stops
        sleep(FIVE_SEC*2);
        
        //ensure there is no running storage after the test
        Assert.assertEquals(getNumberOfMongosRunning(), 0, "there should be no db running after the test");
        
        //ensure there is no running agent after the test
        Assert.assertEquals(getNumberOfAgentsRunning(), 0, "there should not be any agent running after the test");
    }

    public static void main(String[] args)
    {
        new ServiceSmokeTests().doTests(args);
    }

}
